// Adapted by Louise Lessel - 2019
// from
// Author @patriciogv - 2015
// http://patriciogonzalezvivo.com
// https://thebookofshaders.com/02/


/*
Example:
 Color the entire background blue
*/

// These are necessary definitions that let you graphics card know how to render the shader
#ifdef GL_ES
precision mediump float;
#endif

uniform vec2 res;
uniform float time;
uniform sampler2D ramp;
uniform float mouseX;
uniform float mouseY;
uniform float hueOffset;

// Noise
vec3 mod289(vec3 x) { return x - floor(x * (1.0 / 289.0)) * 289.0; }
vec4 mod289(vec4 x) { return x - floor(x * (1.0 / 289.0)) * 289.0; }
vec4 permute(vec4 x) { return mod289(((x*34.0)+1.0)*x); }
vec4 taylorInvSqrt(vec4 r) { return 1.79284291400159 - 0.85373472095314 * r; }
vec3 fade(vec3 t) { return t*t*t*(t*(t*6.0-15.0)+10.0); }
float noise(vec3 P) {
		vec3 i0 = mod289(floor(P)), i1 = mod289(i0 + vec3(1.0));
		vec3 f0 = fract(P), f1 = f0 - vec3(1.0), f = fade(f0);
		vec4 ix = vec4(i0.x, i1.x, i0.x, i1.x), iy = vec4(i0.yy, i1.yy);
		vec4 iz0 = i0.zzzz, iz1 = i1.zzzz;
		vec4 ixy = permute(permute(ix) + iy), ixy0 = permute(ixy + iz0), ixy1 = permute(ixy + iz1);
		vec4 gx0 = ixy0 * (1.0 / 7.0), gy0 = fract(floor(gx0) * (1.0 / 7.0)) - 0.5;
		vec4 gx1 = ixy1 * (1.0 / 7.0), gy1 = fract(floor(gx1) * (1.0 / 7.0)) - 0.5;
		gx0 = fract(gx0); gx1 = fract(gx1);
		vec4 gz0 = vec4(0.5) - abs(gx0) - abs(gy0), sz0 = step(gz0, vec4(0.0));
		vec4 gz1 = vec4(0.5) - abs(gx1) - abs(gy1), sz1 = step(gz1, vec4(0.0));
		gx0 -= sz0 * (step(0.0, gx0) - 0.5); gy0 -= sz0 * (step(0.0, gy0) - 0.5);
		gx1 -= sz1 * (step(0.0, gx1) - 0.5); gy1 -= sz1 * (step(0.0, gy1) - 0.5);
		vec3 g0 = vec3(gx0.x,gy0.x,gz0.x), g1 = vec3(gx0.y,gy0.y,gz0.y),
				g2 = vec3(gx0.z,gy0.z,gz0.z), g3 = vec3(gx0.w,gy0.w,gz0.w),
				g4 = vec3(gx1.x,gy1.x,gz1.x), g5 = vec3(gx1.y,gy1.y,gz1.y),
				g6 = vec3(gx1.z,gy1.z,gz1.z), g7 = vec3(gx1.w,gy1.w,gz1.w);
		vec4 norm0 = taylorInvSqrt(vec4(dot(g0,g0), dot(g2,g2), dot(g1,g1), dot(g3,g3)));
		vec4 norm1 = taylorInvSqrt(vec4(dot(g4,g4), dot(g6,g6), dot(g5,g5), dot(g7,g7)));
		g0 *= norm0.x; g2 *= norm0.y; g1 *= norm0.z; g3 *= norm0.w;
		g4 *= norm1.x; g6 *= norm1.y; g5 *= norm1.z; g7 *= norm1.w;
		vec4 nz = mix(vec4(dot(g0, vec3(f0.x, f0.y, f0.z)), dot(g1, vec3(f1.x, f0.y, f0.z)),
				dot(g2, vec3(f0.x, f1.y, f0.z)), dot(g3, vec3(f1.x, f1.y, f0.z))),
				vec4(dot(g4, vec3(f0.x, f0.y, f1.z)), dot(g5, vec3(f1.x, f0.y, f1.z)),
						dot(g6, vec3(f0.x, f1.y, f1.z)), dot(g7, vec3(f1.x, f1.y, f1.z))), f.z);
		return 2.2 * mix(mix(nz.x,nz.z,f.y), mix(nz.y,nz.w,f.y), f.x);
}
float noise(vec2 P) { return noise(vec3(P, 0.0)); }

float turbulence(vec3 P) {
		float f = 0., s = 1.;
		for (int i = 0 ; i < 9 ; i++) {
				f += abs(noise(s * P)) / s;
				s *= 2.;
				P = vec3(.866 * P.x + .5 * P.z, P.y + 100., -.5 * P.x + .866 * P.z);
		}
		return f;
}

vec3 rgb2hsv(vec3 c)
{
		vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
		vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
		vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

		float d = q.x - min(q.w, q.y);
		float e = 1.0e-10;
		return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

float brightness(vec3 c){
	return sqrt(0.299*(c.r*c.r) + 0.587*(c.g*c.g) + 0.114*(c.b*c.b) );
}

void main() {
		// A blue color
		// In shaders, the RGB color spectrum goes from 0 - 1 instead of 0 - 255
		// vec3 color = vec3(0.0, 0.0, 1.0);

	vec2 uv = vec2(gl_FragCoord.xy) / res;

		vec3 color;
		 float x = gl_FragCoord.x;
		 float y = gl_FragCoord.y;
		 color = vec3(noise(vec3((x + (-mouseX * 0.8)) * 0.003, (y + (-mouseY * 0.8)) * 0.003, time/5000.0)));

		 float pct = 0.0;
	vec2 flow = vec2(noise(vec2(time/5000.0, 0.0)), noise(vec2(time/3000.0, 8.0))) * 500.0;
	pct = distance((vec2(gl_FragCoord.x, gl_FragCoord.y) + flow)/res/2.0, vec2(mouseX, res.y - mouseY)/res);
	vec3 distColor = vec3(1.0 - (pct * 1.05)) * 0.9;
	color = (color * 0.4) + (distColor * 0.6);

	vec3 hsv = rgb2hsv(color);
	vec3 lookup = vec3( texture2D(ramp, vec2(hsv.b) ) );

		// gl_FragColor is a built in shader variable, and you .frag file must contain it
		// We are setting the vec3 color into a new vec4, with an transparency of 1 (no opacity)
		gl_FragColor = vec4(lookup, 1.0);
}
